% SEPTEST Returns 1 if assemblage admits an LHS model, <1 otherwise.
%
%  Based on <a href="http://arxiv.org/abs/1305.1767">arXiv:1305.1767</a>, eq. 2
%
%  val = SEPTEST(assemblage) takes the parameters:
%
%   assemblage: size [$d_B$, $d_B$, $n_A$, $m_A$], collection of sigmas
%
%  And checks Equation (2) in paper.
%
%  Requires a working YALMIP setup
%
%  Latest version at
%  <a href="https://www.mattpusey.uk/steer/">www.mattpusey.co.uk/steer</a>,
%  bug reports welcome at <a href="mailto:m@physics.org">m@physics.org</a>

% Copyright (C) 2013-2018 Matthew F. Pusey
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
% MA 02110-1301, USA.

function val=septest(assemblage)
[~,d,outcomes,settings] = size(assemblage);
strings = outcomes^settings;
powers = outcomes.^(settings-1:-1:0);
F = [];
total = zeros(d);
obtainedassemblage = cell(outcomes*settings);
rho = cell(strings);

for i = 1:(outcomes*settings)
    obtainedassemblage{i} = zeros(d);
end
for i = 1:strings
    string = floor(mod((i-1)./powers,outcomes));
    rho{i} = sdpvar(d,d,'hermitian','complex');
    F = F + [rho{i} >= 0];
    for j = 1:settings
        idx = outcomes*(j-1) + string(j) + 1;
        obtainedassemblage{idx} = obtainedassemblage{idx} + rho{i};
    end
    total = total + rho{i};
end
for outcome = 1:outcomes
    for setting = 1:settings
        idx = outcomes*(setting-1) + outcome;
        F = F + [obtainedassemblage{idx} <= assemblage(:,:,outcome,setting)];
    end
end

thetrace = trace(total);
optimize(F, -thetrace)
val = value(thetrace);