% SRANGE  Calculate the range of a steering inequality for LHS models
%
%  Based on <a href="http://arxiv.org/abs/1305.1767">arXiv:1305.1767</a>, eq. 3
%
%  [maxval,minval] = SRANGE(ineqops) takes the parameter:
%
%   ineqops: size [$d_B$, $d_B$, $n_A$, $m_A$], collection of F matrices
%
%  And attempts to solve Equation (3) plus the corresponding minimization
%
%  Requires a working YALMIP setup
%
%  Latest version at
%  <a href="https://www.mattpusey.uk/steer/">www.mattpusey.co.uk/steer</a>,
%  bug reports welcome at <a href="mailto:m@physics.org">m@physics.org</a>

% Copyright (C) 2013 Matthew F. Pusey
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
% MA 02110-1301, USA.

function [maxval,minval]=srange(ineqops)
[~,d,outcomes,settings] = size(ineqops);
strings = outcomes^settings;
powers = outcomes.^(settings-1:-1:0);
val = 0;
F = [];
total = zeros(d);
rho = cell(strings);

for i = 1:strings
    string = floor(mod((i-1)./powers,outcomes));
    rho{i} = sdpvar(d,d,'hermitian','complex');
    F = F + [rho{i} >= 0];
    for j = 1:settings
        val = val + trace(ineqops(:,:,string(j)+1,j)*rho{i});
    end
    total = total + rho{i};
end

F = F + [trace(total) == 1];
opts = sdpsettings('verbose', 0);

optimize(F, -val,opts)
maxval = value(val);
optimize(F, val,opts)
minval = value(val);
